@@ -25,12 +25,18 @@
 
 #include <openssl/evp.h>
 #include <openssl/err.h>
+#include <openssl/rand.h>
+#include <openssl/pem.h>
 
 #include <assert.h>
 #include <stdlib.h>
 #include <string.h>
 #include <stdbool.h>
 
+#include "utils.h"
+
+#define SALT_PK "UxPlay-Persistent-Not-Secure-Public-Key"
+
 struct aes_ctx_s {
     EVP_CIPHER_CTX *cipher_ctx;
     uint8_t key[AES_128_BLOCK_SIZE];
@@ -261,31 +267,154 @@ void x25519_derive_secret(unsigned char secret[X25519_KEY_SIZE], const x25519_ke
     EVP_PKEY_CTX_free(pctx);
 }
 
+// GCM AES 128
+
+int gcm_encrypt(const unsigned char *plaintext, int plaintext_len, unsigned char *ciphertext,
+                unsigned char *key, unsigned char *iv, unsigned char *tag)
+{
+    EVP_CIPHER_CTX *ctx;
+
+    int len;
+    
+    int ciphertext_len;
+
+    if(!(ctx = EVP_CIPHER_CTX_new()))
+      handle_error(__func__);
+
+    if(1 != EVP_EncryptInit_ex(ctx, EVP_aes_128_gcm(), NULL, NULL, NULL))
+        handle_error(__func__);
+
+    if(1 != EVP_CIPHER_CTX_ctrl(ctx, EVP_CTRL_GCM_SET_IVLEN, 16, NULL))
+        handle_error(__func__);
+
+    if(1 != EVP_EncryptInit_ex(ctx, NULL, NULL, key, iv))
+        handle_error(__func__);
+
+    if(1 != EVP_EncryptUpdate(ctx, ciphertext, &len, plaintext, plaintext_len))
+        handle_error(__func__);
+    ciphertext_len = len;
+
+    if(1 != EVP_EncryptFinal_ex(ctx, ciphertext + len, &len))
+        handle_error(__func__);
+    ciphertext_len += len;
+
+    if(1 != EVP_CIPHER_CTX_ctrl(ctx, EVP_CTRL_GCM_GET_TAG, 16, tag))
+        handle_error(__func__);
+
+    EVP_CIPHER_CTX_free(ctx);
+
+    return ciphertext_len;
+}
+
+int gcm_decrypt(unsigned char *ciphertext, int ciphertext_len, unsigned char *plaintext,
+                unsigned char *key, unsigned char *iv, unsigned char *tag)
+{
+    EVP_CIPHER_CTX *ctx;
+    int len;
+    int plaintext_len;
+    int ret;
+    
+    if(!(ctx = EVP_CIPHER_CTX_new()))
+        handle_error(__func__);
+
+    if(!EVP_DecryptInit_ex(ctx, EVP_aes_128_gcm(), NULL, NULL, NULL))
+        handle_error(__func__);
+
+    if(!EVP_CIPHER_CTX_ctrl(ctx, EVP_CTRL_GCM_SET_IVLEN, 16, NULL))
+        handle_error(__func__);
+
+    if(!EVP_DecryptInit_ex(ctx, NULL, NULL, key, iv))
+        handle_error(__func__);
+
+    if(!EVP_DecryptUpdate(ctx, plaintext, &len, ciphertext, ciphertext_len))
+        handle_error(__func__);
+    plaintext_len = len;
+
+    if(!EVP_CIPHER_CTX_ctrl(ctx, EVP_CTRL_GCM_SET_TAG, 16, tag))
+        handle_error(__func__);
+
+    ret = EVP_DecryptFinal_ex(ctx, plaintext + len, &len);
+
+    EVP_CIPHER_CTX_free(ctx);
+    
+    if(ret > 0) {
+        /* Success */
+        plaintext_len += len;
+        return plaintext_len;
+    } else {
+        /* Verify failed */
+	printf("failed\n");
+        return -1;
+    }
+}
+
 // ED25519
 
 struct ed25519_key_s {
     EVP_PKEY *pkey;
 };
 
-ed25519_key_t *ed25519_key_generate(void) {
+ed25519_key_t *ed25519_key_generate(const char *device_id, const char *keyfile, int *result) {
     ed25519_key_t *key;
     EVP_PKEY_CTX *pctx;
+    BIO *bp;
+    FILE *file;
+    bool new_pk = false;
+    bool use_keyfile = strlen(keyfile);
+
+    *result = 0;
 
     key = calloc(1, sizeof(ed25519_key_t));
     assert(key);
+   
+    if (use_keyfile) {
+        file = fopen(keyfile, "r");
+        if (file) {
+            bp = BIO_new_fp(file, BIO_NOCLOSE);
+            key->pkey = PEM_read_PrivateKey(file, NULL, NULL, NULL);
+            BIO_free(bp);
+            fclose(file);
+            if (!key->pkey) {
+                new_pk = true;
+            }
+        } else {
+            new_pk = true;
+        }
+    } else {
+        /* generate (insecure) persistent keypair using device_id */
+        unsigned char hash[SHA512_DIGEST_LENGTH];
+        char salt[] = SALT_PK;
+        sha_ctx_t *ctx = sha_init();
+        sha_update(ctx, (const unsigned char *) salt, (unsigned int) strlen(salt));
+        sha_update(ctx, (const unsigned char *) device_id, (unsigned int) strlen(device_id));
+        sha_final(ctx, hash, NULL);
+        sha_destroy(ctx);
+        key->pkey = EVP_PKEY_new_raw_private_key(EVP_PKEY_ED25519, NULL, (const unsigned char *) hash, ED25519_KEY_SIZE);
+    }
 
-    pctx = EVP_PKEY_CTX_new_id(EVP_PKEY_ED25519, NULL);
-    if (!pctx) {
-        handle_error(__func__);
+    if (new_pk) {
+        pctx = EVP_PKEY_CTX_new_id(EVP_PKEY_ED25519, NULL);
+        if (!pctx) {
+            handle_error(__func__);
+        }
+        if (!EVP_PKEY_keygen_init(pctx)) {
+            handle_error(__func__);
+        }
+        if (!EVP_PKEY_keygen(pctx, &key->pkey)) {
+            handle_error(__func__);
+        }
+        EVP_PKEY_CTX_free(pctx);
+        if (use_keyfile) {
+            file = fopen(keyfile, "w");
+            if (file) {
+                bp = BIO_new_fp(file, BIO_NOCLOSE);
+                PEM_write_bio_PrivateKey(bp, key->pkey, NULL, NULL, 0, NULL, NULL);
+                BIO_free(bp);
+                fclose(file);
+                *result = 1;
+            }
+        }
     }
-    if (!EVP_PKEY_keygen_init(pctx)) {
-        handle_error(__func__);
-    }
-    if (!EVP_PKEY_keygen(pctx, &key->pkey)) {
-        handle_error(__func__);
-    }
-    EVP_PKEY_CTX_free(pctx);
-
     return key;
 }
 
@@ -423,3 +552,31 @@ void sha_destroy(sha_ctx_t *ctx) {
         free(ctx);
     }
 }
+
+int get_random_bytes(unsigned char *buf, int num) {
+    return RAND_bytes(buf, num);
+}
+#include <stdio.h>
+void pk_to_base64(const unsigned char *pk, int pk_len, char *pk_base64, int len) {
+    memset(pk_base64, 0, len);
+    int len64 = (4 * (pk_len /3)) + (pk_len % 3 ? 4 : 0);
+    
+    assert (len > len64);
+    
+    BIO *b64 = BIO_new(BIO_f_base64());
+    BIO *bio = BIO_new(BIO_s_mem());
+    BUF_MEM *bufferPtr;
+
+
+    bio = BIO_push(b64, bio);
+  
+    BIO_set_flags(bio, BIO_FLAGS_BASE64_NO_NL);
+    BIO_write(bio, pk, pk_len);
+    BIO_flush(bio);
+
+    BIO_get_mem_ptr(bio, &bufferPtr);
+    BIO_set_close(bio, BIO_NOCLOSE);
+    BIO_free_all(bio);
+    memcpy(pk_base64,(*bufferPtr).data, len64);
+}
+  
