@@ -32,18 +32,6 @@
 extern "C" {
 #endif
 
-
-/* Define syslog style log levels */
-#define RAOP_LOG_EMERG       0       /* system is unusable */
-#define RAOP_LOG_ALERT       1       /* action must be taken immediately */
-#define RAOP_LOG_CRIT        2       /* critical conditions */
-#define RAOP_LOG_ERR         3       /* error conditions */
-#define RAOP_LOG_WARNING     4       /* warning conditions */
-#define RAOP_LOG_NOTICE      5       /* normal but significant condition */
-#define RAOP_LOG_INFO        6       /* informational */
-#define RAOP_LOG_DEBUG       7       /* debug-level messages */
-
-
 typedef struct raop_s raop_t;
 
 typedef void (*raop_log_callback_t)(void *cls, int level, const char *msg);
@@ -53,6 +41,8 @@ struct raop_callbacks_s {
 
     void  (*audio_process)(void *cls, raop_ntp_t *ntp, audio_decode_struct *data);
     void  (*video_process)(void *cls, raop_ntp_t *ntp, h264_decode_struct *data);
+    void  (*video_pause)(void *cls);
+    void  (*video_resume)(void *cls);
 
     /* Optional but recommended callback functions */
     void  (*conn_init)(void *cls);
@@ -68,12 +58,17 @@ struct raop_callbacks_s {
     void  (*audio_set_progress)(void *cls, unsigned int start, unsigned int curr, unsigned int end);
     void  (*audio_get_format)(void *cls, unsigned char *ct, unsigned short *spf, bool *usingScreen, bool *isMedia, uint64_t *audioFormat);
     void  (*video_report_size)(void *cls, float *width_source, float *height_source, float *width, float *height);
+    void  (*report_client_request) (void *cls, char *deviceid, char *model, char *name, bool *admit);
+    void  (*display_pin) (void *cls, char * pin);
+    void  (*register_client) (void *cls, const char *device_id, const char *pk_str, const char *name);
+    bool  (*check_register) (void *cls, const char *pk_str);
+    void  (*export_dacp) (void *cls, const char *active_remote, const char *dacp_id);
 };
 typedef struct raop_callbacks_s raop_callbacks_t;
-raop_ntp_t *raop_ntp_init(logger_t *logger, raop_callbacks_t *callbacks, const unsigned char *remote_addr, int remote_addr_len,
+raop_ntp_t *raop_ntp_init(logger_t *logger, raop_callbacks_t *callbacks, const char *remote, int remote_addr_len,
                           unsigned short timing_rport, timing_protocol_t *time_protocol);
 
-RAOP_API raop_t *raop_init(int max_clients, raop_callbacks_t *callbacks);
+RAOP_API raop_t *raop_init(int max_clients, raop_callbacks_t *callbacks, const char *device_id, const char *keyfile);
 RAOP_API void raop_set_log_level(raop_t *raop, int level);
 RAOP_API void raop_set_log_callback(raop_t *raop, raop_log_callback_t callback, void *cls);
 RAOP_API int raop_set_plist(raop_t *raop, const char *plist_item, const int value);
