@@ -107,9 +107,8 @@ struct raop_rtp_mirror_s {
 };
 
 static int
-raop_rtp_parse_remote(raop_rtp_mirror_t *raop_rtp_mirror, const unsigned char *remote, int remotelen)
+raop_rtp_mirror_parse_remote(raop_rtp_mirror_t *raop_rtp_mirror, const char *remote, int remotelen)
 {
-    char current[25];
     int family;
     int ret;
     assert(raop_rtp_mirror);
@@ -120,10 +119,8 @@ raop_rtp_parse_remote(raop_rtp_mirror_t *raop_rtp_mirror, const unsigned char *r
     } else {
         return -1;
     }
-    memset(current, 0, sizeof(current));
-    snprintf(current, sizeof(current), "%d.%d.%d.%d", remote[0], remote[1], remote[2], remote[3]);
-    logger_log(raop_rtp_mirror->logger, LOGGER_DEBUG, "raop_rtp_mirror parse remote ip = %s", current);
-    ret = netutils_parse_address(family, current,
+    logger_log(raop_rtp_mirror->logger, LOGGER_DEBUG, "raop_rtp_mirror parse remote ip = %s", remote);
+    ret = netutils_parse_address(family, remote,
                                  &raop_rtp_mirror->remote_saddr,
                                  sizeof(raop_rtp_mirror->remote_saddr));
     if (ret < 0) {
@@ -135,7 +132,7 @@ raop_rtp_parse_remote(raop_rtp_mirror_t *raop_rtp_mirror, const unsigned char *r
 
 #define NO_FLUSH (-42)
 raop_rtp_mirror_t *raop_rtp_mirror_init(logger_t *logger, raop_callbacks_t *callbacks, raop_ntp_t *ntp,
-                                        const unsigned char *remote, int remotelen, const unsigned char *aeskey)
+                                        const char *remote, int remotelen, const unsigned char *aeskey)
 {
     raop_rtp_mirror_t *raop_rtp_mirror;
 
@@ -155,7 +152,7 @@ raop_rtp_mirror_t *raop_rtp_mirror_init(logger_t *logger, raop_callbacks_t *call
         free(raop_rtp_mirror);
         return NULL;
     }
-    if (raop_rtp_parse_remote(raop_rtp_mirror, remote, remotelen) < 0) {
+    if (raop_rtp_mirror_parse_remote(raop_rtp_mirror, remote, remotelen) < 0) {
         free(raop_rtp_mirror);
         return NULL;
     }
@@ -198,6 +195,7 @@ raop_rtp_mirror_thread(void *arg)
     uint64_t ntp_timestamp_local  = 0;
     unsigned char nal_start_code[4] = { 0x00, 0x00, 0x00, 0x01 };
     bool logger_debug = (logger_get_level(raop_rtp_mirror->logger) >= LOGGER_DEBUG);
+    bool h265_video_detected = false;
 
     while (1) {
         fd_set rfds;
@@ -206,7 +204,7 @@ raop_rtp_mirror_thread(void *arg)
         MUTEX_LOCK(raop_rtp_mirror->run_mutex);
         if (!raop_rtp_mirror->running) {
             MUTEX_UNLOCK(raop_rtp_mirror->run_mutex);
-            logger_log(raop_rtp_mirror->logger, LOGGER_ERR, "raop_rtp_mirror->running is no longer true");
+            logger_log(raop_rtp_mirror->logger, LOGGER_INFO, "raop_rtp_mirror->running is no longer true");
             break;
         }
         MUTEX_UNLOCK(raop_rtp_mirror->run_mutex);
@@ -292,7 +290,7 @@ raop_rtp_mirror_thread(void *arg)
             }
 
             if (payload == NULL && ret == 0) {
-                logger_log(raop_rtp_mirror->logger, LOGGER_ERR,
+                logger_log(raop_rtp_mirror->logger, LOGGER_DEBUG,
                            "raop_rtp_mirror tcp socket is closed, got %d bytes of 128 byte header",readstart);
                 FD_CLR(stream_fd, &rfds);
                 stream_fd = -1;
@@ -445,7 +443,22 @@ raop_rtp_mirror_thread(void *arg)
                         break;
                     }
                     int nalu_type = payload_decrypted[nalu_size] & 0x1f;
-                    int ref_idc = (payload_decrypted[nalu_size] >> 5);		    
+                    int ref_idc = (payload_decrypted[nalu_size] >> 5);
+                    /* check for unsupported h265 video (sometimes sent by macOS in high-def screen mirroring) */
+                    if (payload_decrypted[nalu_size + 1] == 0x01) {
+                        switch (payload_decrypted[nalu_size]) {
+                        case 0x28:    // h265 IDR type 20 NAL
+                        case 0x02:    // h265 non-IDR type 1 NAL
+                            ref_idc = 0;
+                            h265_video_detected = true;
+                            break;
+                        default:
+                            break;
+                        }
+                        if (h265_video_detected) {
+                            break;
+                        }
+                    }
                     switch (nalu_type) {
                     case 14:  /* Prefix NALu , seen before all VCL Nalu's in AirMyPc */
                     case 5:   /*IDR, slice_layer_without_partitioning */
@@ -495,6 +508,12 @@ raop_rtp_mirror_thread(void *arg)
 		    }
                     nalu_size += nc_len;
                 }
+                if (h265_video_detected) {
+                    logger_log(raop_rtp_mirror->logger, LOGGER_ERR,
+                               "unsupported h265 video detected");
+                    free (payload_out);
+                    break;
+                }
                 if (nalu_size != payload_size) valid_data = false;
                 if(!valid_data) {
                     logger_log(raop_rtp_mirror->logger, LOGGER_DEBUG, "nalu marked as invalid");
@@ -513,6 +532,7 @@ raop_rtp_mirror_thread(void *arg)
                     h264_data.nal_count += 2;
 		    prepend_sps_pps =  false;
                 }
+                raop_rtp_mirror->callbacks.video_resume(raop_rtp_mirror->callbacks.cls);
                 raop_rtp_mirror->callbacks.video_process(raop_rtp_mirror->callbacks.cls, raop_rtp_mirror->ntp, &h264_data);
                 free(payload_out);
                 break;
@@ -603,7 +623,7 @@ raop_rtp_mirror_thread(void *arg)
                 // h264.pps_size = pps_size;
                 // h264.picture_parameter_set = malloc(h264.pps_size);
                 // memcpy(h264.picture_parameter_set, picture_parameter_set, pps_size);
-
+                raop_rtp_mirror->callbacks.video_pause(raop_rtp_mirror->callbacks.cls);
                 break;
             case 0x02:
                 logger_log(raop_rtp_mirror->logger, LOGGER_DEBUG, "\nReceived old-protocol once-per-second packet from client:"
@@ -704,7 +724,7 @@ raop_rtp_init_mirror_sockets(raop_rtp_mirror_t *raop_rtp_mirror, int use_ipv6)
 }
 
 void
-raop_rtp_start_mirror(raop_rtp_mirror_t *raop_rtp_mirror, int use_udp, unsigned short *mirror_data_lport,
+raop_rtp_start_mirror(raop_rtp_mirror_t *raop_rtp_mirror, unsigned short *mirror_data_lport,
                       uint8_t show_client_FPS_data)
 {
     logger_log(raop_rtp_mirror->logger, LOGGER_INFO, "raop_rtp_mirror starting mirroring");
@@ -723,7 +743,7 @@ raop_rtp_start_mirror(raop_rtp_mirror_t *raop_rtp_mirror, int use_udp, unsigned
     if (raop_rtp_mirror->remote_saddr.ss_family == AF_INET6) {
         use_ipv6 = 1;
     }
-    use_ipv6 = 0;
+    //use_ipv6 = 0;
      
     raop_rtp_mirror->mirror_data_lport = *mirror_data_lport;
     if (raop_rtp_init_mirror_sockets(raop_rtp_mirror, use_ipv6) < 0) {
