@@ -10,6 +10,9 @@
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * Lesser General Public License for more details.
+ *
+ *=================================================================
+ * modified by fduncanh 2022-2023
  */
 
 #ifndef AIRPLAYSERVER_STREAM_H
@@ -22,13 +25,17 @@ typedef struct {
     int nal_count;
     unsigned char *data;
     int data_len;
-    uint64_t ntp_time;
+    uint64_t ntp_time_local;
+    uint64_t ntp_time_remote;
 } h264_decode_struct;
 
 typedef struct {
     unsigned char *data;
+    unsigned char ct;
     int data_len;
-    uint64_t ntp_time;
+    int sync_status;
+    uint64_t ntp_time_local;
+    uint64_t ntp_time_remote;
     uint64_t rtp_time;
     unsigned short seqnum;
 } audio_decode_struct;
