@@ -52,6 +52,9 @@ void video_renderer_init (logger_t *logger, const char *server_name, videoflip_t
                           const bool *video_sync);
 void video_renderer_start ();
 void video_renderer_stop ();
+void video_renderer_pause ();
+void video_renderer_resume ();
+bool video_renderer_is_paused();
 void video_renderer_render_buffer (unsigned char* data, int *data_len, int *nal_count, uint64_t *ntp_time);
 void video_renderer_flush ();
 unsigned int video_renderer_listen(void *loop);
